\\ Copyright 2016 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


\\-----------------------------------------------------------------------------

     \\ if den has common factor in its exponents like 1-2*x^6-x^12 then
     \\ set spread=6 and reduce to den=1-2*x-x^2
     my(spread=gcd(vector(poldegree(den,'x),i, if(polcoeff(den,i,'x),i,0))));
     den=substpol(den,'x^spread,'x);
     if(debug, print(" spread "spread"\n den "den));

\\ p is a polynomial.  Return the gcd of the powers which occur.
\\ For example p=x^15+x^9+x^6+1 gives 3.
recurrence_guess_INTERNAL_pol_powers_gcd(p) = \
  gcd(vector(#p,i,i--; if(polcoeff(p,i),i,0)));


\\-----------------------------------------------------------------------------

\\ return a generating function for f(n) = n^expon
recurrence_guess_INTERNAL_expon_to_gf(expon) =
{
  if(expon==0,
     1/(1-x),
     Pol(vector(expon+1,k, sum(j=0,k, (-1)^j*(k-j)^expon*binomial(expon+1, j))))
     / (1-x)^(expon+1));
}

{
  for(expon=0,10,
     my(want=vector(20,k,k--; k^expon));
     my(g=recurrence_guess_INTERNAL_expon_to_gf(expon));
     my(got=recurrence_guess_INTERNAL_gf_terms(g,20));
     test_equal(got,want, Str("expon=",expon)));
}

  my(debug=0);
  my(m=lcm(spread,#periodic_vector));
  spread*=m;
  base^=
  periodic_vector=vector(spread,i,i--;
                         recurrence_guess_INTERNAL_vector_modulo(periodic_vector,i)),
  \\ if(debug,
  \\    print(" spread "spread),
  \\    print(" periodic_vector "periodic_vector));
  \\ my(g=0);
  \\ for(i=1,#periodic_vector,
  \\    my(want='x^expon * periodic_vector[i]);
  \\    if(debug,print(" want "want));
  \\    forstep(e=expon,0, -1,
  \\           my(p=prod(k=1,e, ('x-(i-1) + k)) / e!,
  \\              f=polcoeff(want,e,'x) / polcoeff(p,e,'x));
  \\           want -= f*p;
  \\           g += 'x^(i-1)/(1 - base*'x^spread)^(e+1)));
  \\ return(g);
    
  

\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_pol_is_high_only()

test_equal(recurrence_guess_INTERNAL_pol_is_high_only(1+x), 1, "1+x");
test_equal(recurrence_guess_INTERNAL_pol_is_high_only(1+x^5), 1);
test_equal(recurrence_guess_INTERNAL_pol_is_high_only(1+x+x^5), 0);
test_equal(recurrence_guess_INTERNAL_pol_is_high_only(1+x^4+x^5), 0);

\\ Return true if polynomial p is of the form A*x^n + B, some n>=1,
\\ so nothing but high term and constant.
recurrence_guess_INTERNAL_pol_is_high_only(p) =
{
  if(poldegree(p) < 1, return(0));
  my(v = Vec(p));
  for(i=2,#v-1, if(v[i],return(0)));
  1;
}



\\-----------------------------------------------------------------------------
     \\ prefer positive base
     if(real(base) < 0 || (real(base)==0 && imag(base) < 0),
        base = -base;
        period *= 2;
        if(debug, print(" base "base"  period "period)));
     
     \\ base will be p-th root of c, in the number type of c (complex or quad)
     \\ try also to work some of spread into p
     my(base=0);
     my(spread_divisors=divisors(spread));
     spread_divisors=[1];
     forstep(s=#spread_divisors,1, -1,  \\ highest to lowest divisor
            my(sd=spread_divisors[s],
               f=factor(subst(periodic_pol,'x,'x^sd)));
            if(debug, print(" sd="sd" factors ",f));
            for(i=1,matsize(f)[1],
               my(p=f[i,1]);
               if(poldegree(p,'x)==1,
                  base=polcoeff(p,1,'x) / polcoeff(p,0,'x);
                  spread /= sd;
                  break(2))));
     if(debug, print(" base ",base,"   spread "spread));
     if(base==0,
        \\ no exact nth-root of periodic_pol high coefficient,
        \\ so no exact base for powers base^n
        listput(glist,gpart);
        next());

\\-----------------------------------------------------------------------------


  \\ recurrence_guess_INTERNAL_arg_0_to_2pi(z) = my(a=arg(z)); if(a<0,a+=2*Pi); a;
  \\ my(abs_a=abs(a), abs_b=abs(b));
  \\ if(abs_a == abs_b && abs_a != 0,
  \\    lex(recurrence_guess_INTERNAL_arg_0_to_2pi(a),
  \\        recurrence_guess_INTERNAL_arg_0_to_2pi(b)),
  \\    lex(abs_a,abs_b));
}
